@@ -9,6 +9,7 @@ import shutil
 import stat
 import subprocess
 import sys
+import zipfile
 from collections import OrderedDict
 from itertools import product
 from stat import S_IREAD, S_IRGRP, S_IROTH
@@ -19,9 +20,10 @@ import pytest
 
 from virtualenv.__main__ import run, run_with_catch
 from virtualenv.create.creator import DEBUG_SCRIPT, Creator, get_env_debug_info
+from virtualenv.create.via_global_ref.builtin.python2.python2 import Python2
 from virtualenv.discovery.builtin import get_interpreter
 from virtualenv.discovery.py_info import PythonInfo
-from virtualenv.info import IS_PYPY, IS_WIN, PY3, fs_is_case_sensitive, fs_supports_symlink
+from virtualenv.info import IS_PYPY, IS_WIN, PY2, PY3, fs_is_case_sensitive
 from virtualenv.pyenv_cfg import PyEnvCfg
 from virtualenv.run import cli_run, session_via_cli
 from virtualenv.util.path import Path
@@ -59,6 +61,10 @@ def test_destination_exists_file(tmp_path, capsys):
 
 @pytest.mark.skipif(sys.platform == "win32", reason="Windows only applies R/O to files")
 def test_destination_not_write_able(tmp_path, capsys):
+    if hasattr(os, "geteuid"):
+        if os.geteuid() == 0:
+            pytest.skip("no way to check permission restriction when running under root")
+
     target = tmp_path
     prev_mod = target.stat().st_mode
     target.chmod(S_IREAD | S_IRGRP | S_IROTH)
@@ -88,6 +94,13 @@ def system(session_app_data):
 
 
 CURRENT_CREATORS = list(i for i in CURRENT.creators().key_to_class.keys() if i != "builtin")
+CREATE_METHODS = []
+for k, v in CURRENT.creators().key_to_meta.items():
+    if k in CURRENT_CREATORS:
+        if v.can_copy:
+            CREATE_METHODS.append((k, "copies"))
+        if v.can_symlink:
+            CREATE_METHODS.append((k, "symlinks"))
 _VENV_BUG_ON = (
     IS_PYPY
     and CURRENT.version_info[0:3] == (3, 6, 9)
@@ -97,7 +110,7 @@ _VENV_BUG_ON = (
 
 
 @pytest.mark.parametrize(
-    "creator, method, isolated",
+    "creator, isolated",
     [
         pytest.param(
             *i,
@@ -106,15 +119,15 @@ _VENV_BUG_ON = (
                 strict=True,
             )
         )
-        if _VENV_BUG_ON and i[0] == "venv" and i[1] == "copies"
+        if _VENV_BUG_ON and i[0][0] == "venv" and i[0][1] == "copies"
         else i
-        for i in product(
-            CURRENT_CREATORS, (["copies"] + (["symlinks"] if fs_supports_symlink() else [])), ["isolated", "global"]
-        )
+        for i in product(CREATE_METHODS, ["isolated", "global"])
     ],
+    ids=lambda i: "-".join(i) if isinstance(i, tuple) else i,
 )
-def test_create_no_seed(python, creator, isolated, system, coverage_env, special_name_dir, method):
+def test_create_no_seed(python, creator, isolated, system, coverage_env, special_name_dir):
     dest = special_name_dir
+    creator_key, method = creator
     cmd = [
         "-v",
         "-v",
@@ -125,7 +138,7 @@ def test_create_no_seed(python, creator, isolated, system, coverage_env, special
         "--activators",
         "",
         "--creator",
-        creator,
+        creator_key,
         "--{}".format(method),
     ]
     if isolated == "global":
@@ -187,12 +200,12 @@ def test_create_no_seed(python, creator, isolated, system, coverage_env, special
         exes = ("python.exe",)
     else:
         exes = ("python", "python{}".format(*sys.version_info), "python{}.{}".format(*sys.version_info))
-        # pypy3<=7.3: https://bitbucket.org/pypy/pypy/pull-requests/697
-        if IS_PYPY and CURRENT.pypy_version_info[:3] <= [7, 3, 0] and creator == "venv":
+        if creator_key == "venv":
+            # for venv some repackaging does not includes the pythonx.y
             exes = exes[:-1]
     for exe in exes:
         exe_path = result.creator.bin_dir / exe
-        assert exe_path.exists()
+        assert exe_path.exists(), "\n".join(str(i) for i in result.creator.bin_dir.iterdir())
         if not exe_path.is_symlink():  # option 1: a real file
             continue  # it was a file
         link = os.readlink(str(exe_path))
@@ -209,6 +222,10 @@ def test_create_no_seed(python, creator, isolated, system, coverage_env, special
 
 @pytest.mark.skipif(not CURRENT.has_venv, reason="requires interpreter with venv")
 def test_venv_fails_not_inline(tmp_path, capsys, mocker):
+    if hasattr(os, "geteuid"):
+        if os.geteuid() == 0:
+            pytest.skip("no way to check permission restriction when running under root")
+
     def _session_via_cli(args, options=None):
         session = session_via_cli(args, options)
         assert session.creator.can_be_inline is False
@@ -296,7 +313,7 @@ def cross_python(is_inside_ci, session_app_data):
 
 
 @pytest.mark.slow
-def test_cross_major(cross_python, coverage_env, tmp_path, current_fastest, session_app_data):
+def test_cross_major(cross_python, coverage_env, tmp_path, session_app_data, current_fastest):
     cmd = [
         "-v",
         "-v",
@@ -307,13 +324,16 @@ def test_cross_major(cross_python, coverage_env, tmp_path, current_fastest, sess
         "--no-wheel",
         "--activators",
         "",
-        "--creator",
-        current_fastest,
     ]
     result = cli_run(cmd)
     pip_scripts = {i.name.replace(".exe", "") for i in result.creator.script_dir.iterdir() if i.name.startswith("pip")}
     major, minor = cross_python.version_info[0:2]
-    assert pip_scripts == {"pip", "pip-{}.{}".format(major, minor), "pip{}".format(major)}
+    assert pip_scripts == {
+        "pip",
+        "pip{}".format(major),
+        "pip-{}.{}".format(major, minor),
+        "pip{}.{}".format(major, minor),
+    }
     coverage_env()
     env = PythonInfo.from_exe(str(result.creator.exe), session_app_data)
     assert env.version_info.major != CURRENT.version_info.major
@@ -384,14 +404,33 @@ def test_create_distutils_cfg(creator, tmp_path, monkeypatch):
 
     monkeypatch.chdir(dest)  # distutils will read the setup.cfg from the cwd, so change to that
 
-    install_demo_cmd = [str(result.creator.script("pip")), "install", str(dest), "--no-use-pep517"]
+    install_demo_cmd = [
+        str(result.creator.script("pip")),
+        "--disable-pip-version-check",
+        "install",
+        str(dest),
+        "--no-use-pep517",
+        "-vv",
+    ]
     subprocess.check_call(install_demo_cmd)
 
     magic = result.creator.script("magic")  # console scripts are created in the right location
     assert magic.exists()
 
-    package_folder = result.creator.platlib / "demo"  # prefix is set to the virtualenv prefix for install
-    assert package_folder.exists()
+    package_folder = result.creator.purelib / "demo"  # prefix is set to the virtualenv prefix for install
+    assert package_folder.exists(), list_files(str(tmp_path))
+
+
+def list_files(path):
+    result = ""
+    for root, _, files in os.walk(path):
+        level = root.replace(path, "").count(os.sep)
+        indent = " " * 4 * level
+        result += "{}{}/\n".format(indent, os.path.basename(root))
+        sub = " " * 4 * (level + 1)
+        for f in files:
+            result += "{}{}\n".format(sub, f)
+    return result
 
 
 @pytest.mark.parametrize("python_path_on", [True, False], ids=["on", "off"])
@@ -448,3 +487,52 @@ def test_python_path(monkeypatch, tmp_path, python_path_on):
         assert non_python_path == [i for i in base if i not in extra_as_python_path]
     else:
         assert base == extra_all
+
+
+@pytest.mark.skipif(
+    not (CURRENT.implementation == "CPython" and PY2),
+    reason="stdlib components without py files only possible on CPython2",
+)
+def test_pyc_only(tmp_path, mocker, session_app_data):
+    """Ensure that creation can succeed if os.pyc exists (even if os.py has been deleted)"""
+    interpreter = PythonInfo.from_exe(sys.executable, session_app_data)
+    host_pyc, _, host_pyc_exists = Python2.from_stdlib(Python2.mappings(interpreter), "os.pyc")
+    if not host_pyc_exists:
+        pytest.skip("missing system os.pyc at {}".format(host_pyc))
+    previous = Python2.from_stdlib
+
+    def from_stdlib(mappings, name):
+        path, to, exists = previous(mappings, name)
+        if name.endswith(".py"):
+            exists = False
+        return path, to, exists
+
+    mocker.patch.object(Python2, "from_stdlib", side_effect=from_stdlib)
+
+    result = cli_run([ensure_text(str(tmp_path)), "--without-pip", "--activators", ""])
+
+    assert not (result.creator.stdlib / "os.py").exists()
+    assert (result.creator.stdlib / "os.pyc").exists()
+    assert "os.pyc" in result.creator.debug["os"]
+
+
+def test_zip_importer_can_import_setuptools(tmp_path):
+    """We're patching the loaders so might fail on r/o loaders, such as zipimporter on CPython<3.8"""
+    result = cli_run([str(tmp_path / "venv"), "--activators", "", "--no-pip", "--no-wheel", "--copies"])
+    zip_path = tmp_path / "site-packages.zip"
+    with zipfile.ZipFile(str(zip_path), "w", zipfile.ZIP_DEFLATED) as zip_handler:
+        lib = str(result.creator.purelib)
+        for root, _, files in os.walk(lib):
+            base = root[len(lib) :].lstrip(os.pathsep)
+            for file in files:
+                if not file.startswith("_virtualenv"):
+                    zip_handler.write(filename=os.path.join(root, file), arcname=os.path.join(base, file))
+    for folder in result.creator.purelib.iterdir():
+        if not folder.name.startswith("_virtualenv"):
+            if folder.is_dir():
+                shutil.rmtree(str(folder), ignore_errors=True)
+            else:
+                folder.unlink()
+    env = os.environ.copy()
+    env[str("PYTHONPATH")] = str(zip_path)
+    subprocess.check_call([str(result.creator.exe), "-c", "from setuptools.dist import Distribution"], env=env)
